;                     VEMS
; spmlocal.S
; assembly language bootloader upgrade commands for the Genboard ATMEGA128
; ATMEGA 128: SPMCSR 0x68
; ATMEGA 168: SPMCSR 0x57

.section .spmsect_local,"ax",@progbits
.global page_erase_local
page_erase_local:                 ;
        RCALL   WAIT_SPMEN_local  ;Wait for SPMEN flag cleared
        MOVW    R30,R24     ;move address to z pointer (R31=ZH R30=ZL)
        LDI     R18,0x03    ; 3 = page erase, data sheet pg 278
        STS 	0x68, R18   ; load spmcsr with page erase command
        SPM                 ;erase the page
        RET
.global page_write_local
page_write_local:
        RCALL   WAIT_SPMEN_local  ;Wait for SPMEN flag cleared
        MOVW    R30,R24     ;move address to z pointer (R31=ZH R30=ZL)
        LDI     R18,0x05    ; 5 = page write, data sheet pg 277
        STS 	0x68, R18   ; load spmcsr with page write command
        SPM                 ;write the page
        RET

; the Z-pointer not needed here
; according  to pg 273 boot loader lock bits will be set according to R0
.global reenable_rww_local
reenable_rww_local:
        RCALL   WAIT_SPMEN_local  ;Wait for SPMEN flag cleared
        LDI     R18,0x11    ; 0x11 = re-enable rww, data sheet pg 277
        STS 	0x68, R18   ; load spmcsr with re-enable command
        SPM                 ;do the re-enable
        RET

.global fill_temp_buffer_local
fill_temp_buffer_local:           ; (data,address);
        RCALL   WAIT_SPMEN_local  ;Wait for SPMEN flag cleared
        MOVW    R30,R22     ;move address to z pointer (R31=ZH R30=ZL)
        MOVW    R0,R24      ;move data to reg 0 and 1
        LDI     R18,0x01    ; (uint8_t)SPMEN_BIT
        STS 	0x68, R18   ;load spmcsr with buffer fill command
        SPM                 ;Store the data
        EOR     R1, R1      ;restore zero
        RET


.global read_prog_mem_word_local
read_prog_mem_word_local:
        RCALL   WAIT_SPMEN_local
        MOVW    R30,R24     ;move address to z pointer
        ELPM    R24,Z+      ;read LSB  use LPM's for smaller mega parts
        ELPM    R25,Z+      ;read MSB
        RET

; would work with one instruction 0x37 instead of 0x57
WAIT_SPMEN_local:
        LDS     R18,0x68        ;get SPMCSR register
	SBRC    R18,0           ;test the SPMEN bit
	RJMP    WAIT_SPMEN_local      ;Wait for SPMEN bit cleared
	RET

.global set_blb_lockbits
set_blb_lockbits:
        RCALL   WAIT_SPMEN_local  ;Wait for SPMEN flag cleared
        MOVW    R0,R24     ;move value to R0
        LDI     R18,0x09    ; 9 = blbset, data sheet pg 273
        STS 	0x68, R18   ; load spmcsr with re-enable command
        SPM                 ;do the re-enable
        RET

.global read_lock_byte
read_lock_byte:
        RCALL   WAIT_SPMEN_local  ;Wait for SPMEN flag cleared
        LDI     R30,0x01     ;move address to z pointer (R31=ZH R30=ZL)
        LDI     R31,0x00     ;move address to z pointer (R31=ZH R30=ZL)
        LDI     R18,0x09    ; (uint8_t)SPMEN_BIT | BLBSET bit
        STS 	0x68, R18   ;load spmcsr with buffer fill command
        LPM     ;read lock byte
        MOV     R24,R0
        RET

; fill R0 as you like
;.global set_blb_lockbits
;set_blb_lockbits:
;        RCALL   WAIT_SPMEN  ;Wait for SPMEN flag cleared
;        LDI     R18,9    ; 9 = blbset, data sheet pg 273
;        STS 	0x68, R18   ; load spmcsr with re-enable command
;        SPM                 ;do the re-enable
;        RET
